@@ -1474,22 +1474,6 @@ main(int argc, char *argv[])
   gtk_widget_push_visual(gdk_imlib_get_visual());
   gtk_widget_push_colormap(gdk_imlib_get_colormap());
   gnome_win_hints_init();
-  if (!gnome_win_hints_wm_exists())
-    {
-      GtkWidget *d, *l;
-      
-      d = gnome_error_dialog(_("Gnome Pager Error"));
-      l = gtk_label_new(_("You are not running a GNOME Compliant Window Manager.\n"
-			  "Please run a GNOME Compliant Window Manager.\n"
-			  "For Example:\n"
-			  "Enlightenment (DR-0.15)\n"
-			  "Then start this applet again.\n"));
-      gtk_widget_show(l);
-      gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox), l, TRUE, TRUE, 5);
-      gnome_dialog_run(GNOME_DIALOG(d));
-      
-      gtk_exit(1);
-    }
 
   /*FIXME: remove this later!!!!!, in favour of the WELL BEHAVED
     session saving/loading, leave it in for now so that people that
@@ -1521,6 +1505,26 @@ main(int argc, char *argv[])
       g_error("Can't create applet!\n");
       exit(1);
     }
+
+  if (!gnome_win_hints_wm_exists()) {
+    GtkWidget *d, *l;
+    
+    d = gnome_error_dialog(_("Gnome Pager Error"));
+    l = gtk_label_new(_("You are not running a GNOME Compliant "
+			"Window Manager.\n"
+			"Please run a GNOME Compliant "
+			"Window Manager.\n"
+			"For Example:\n"
+			"Enlightenment (DR-0.15)\n"
+			"Then start this applet again.\n"));
+    gtk_widget_show(l);
+    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox), l,
+		       TRUE, TRUE, 5);
+    gnome_dialog_run(GNOME_DIALOG(d));
+    applet_widget_remove (APPLET_WIDGET(applet));
+    gtk_exit(1);
+  }
+
   gtk_signal_connect(GTK_OBJECT(applet), "change_orient",
 		     GTK_SIGNAL_FUNC(cb_applet_orient_change),
 		     NULL);
