@@ -32,6 +32,11 @@
 #include "macro.h"
 #include "message.h"
 
+/*in case SA_NOMASK doesn't exist, use SA_NODEFER*/
+#ifndef SA_NOMASK
+#define SA_NOMASK SA_NODEFER
+#endif
+
 /* 
    These routines will only work on UNIX-alike system because fork()
    is needed. Perhaps I will add an alternative for non UNIX system in
@@ -44,7 +49,7 @@ void
 execCommand(char *cmd)
 {
     pid_t pid;
-    char *argv[10];
+    char *argv[5];
     char command[1000];
     pid_t PID = getpid();
 	    
@@ -62,6 +67,15 @@ execCommand(char *cmd)
     else if (pid == 0) 
 	{
 	    /* child */
+	    /* try the bash as interactive login so that personal prolfiles are reflected */
+/* 	    argv[0] = "bash"; */
+/* 	    argv[1] = "--login"; */
+/* 	    argv[2] = "-c"; */
+/* 	    argv[3] = command; */
+/* 	    argv[4] = NULL;   */
+/* 	    execv("/bin/sh", argv); */
+
+	    /* looks like there is no bash -> try sh */
 	    argv[0] = "sh";
 	    argv[1] = "-c";
 	    argv[2] = command;
@@ -83,16 +97,16 @@ execCommand(char *cmd)
 static
 void sighandle_sigchld(int sig)
 {
-    pid_t pid = 0;
-    int status;
+    int show_message = 0;
 
     /* call waitpid to remove the child process and to prevent that it
        becomes a zombie */
-    if (waitpid(0, &status, WNOHANG ) > 0)
-	showMessage((gchar *) _("child exited")); 
 
-    /* reinstall signal handler */
-    signal(SIGCHLD, &sighandle_sigchld);
+    while(waitpid(0, NULL, WNOHANG) > 0)
+	show_message = 1;
+
+    if(show_message)
+	showMessage((gchar *) _("child exited"));
 }
 
 static
@@ -113,15 +127,27 @@ void sighandle_sigalrm(int sig)
        NULL)
        )
        ); */
-
-    /* reinstall signal handler */
-    signal(SIGALRM, &sighandle_sigalrm);
 }
 
 void
 initExecSignalHandler(void)
 {
-    /* install signal handlers */
-    signal(SIGCHLD, &sighandle_sigchld);
-    signal(SIGALRM, &sighandle_sigalrm);
+    struct sigaction act = {0}; /*this sets all fields to 0, no
+				  matter how many fields there are*/
+
+    act.sa_handler= &sighandle_sigchld;
+    /* act.sa_mask =  */
+    act.sa_flags = SA_NOMASK;
+    /* act.sa_restorer = NULL; */
+
+    /* install signal handler */
+    sigaction(SIGCHLD, &act, NULL); 
+
+    act.sa_handler= &sighandle_sigalrm;
+    /* act.sa_mask =  */
+    act.sa_flags = SA_NOMASK;
+    /* act.sa_restorer = NULL; */
+
+    /* install signal handler */
+    sigaction(SIGALRM, &act, NULL); 
 }
