@@ -486,6 +486,9 @@ applet_widget_register_callback(AppletWidget *applet,
 {
 	g_return_if_fail(applet != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
+	g_return_if_fail(name != NULL);
+	g_return_if_fail(menutext != NULL);
+	g_return_if_fail(func != NULL);
 
 	gnome_panel_applet_register_callback (GTK_WIDGET(applet),name,
 					      "",menutext,func,data);
@@ -501,6 +504,10 @@ applet_widget_register_stock_callback(AppletWidget *applet,
 {
 	g_return_if_fail(applet != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
+	g_return_if_fail(name != NULL);
+	g_return_if_fail(stock_type != NULL);
+	g_return_if_fail(menutext != NULL);
+	g_return_if_fail(func != NULL);
 
 	gnome_panel_applet_register_callback (GTK_WIDGET(applet),name,
 					      stock_type,menutext,func,data);
@@ -852,7 +859,8 @@ applet_widget_init(const char *app_id,
 
 	gnome_client_disable_master_connection ();
 	CORBA_exception_init(&ev);
-	orb = gnome_CORBA_init_with_popt_table(app_id, VERSION, &argc, argv,
+	orb = gnome_CORBA_init_with_popt_table(app_id, app_version, 
+					       &argc, argv,
 					       options, flags, return_ctx,
 					       GNORBA_INIT_SERVER_FUNC, &ev);
 
