@@ -9,6 +9,7 @@
 #include <config.h>
 #include <string.h>
 #include <signal.h>
+#include <limits.h>
 #include <gnome.h>
 #include <gdk/gdkx.h>
 
@@ -480,8 +481,6 @@ do_session_save(GnomeClient *client,
 				      global_config.hungry_menus);
 		gnome_config_set_bool("off_panel_popups",
 				      global_config.off_panel_popups);
-		gnome_config_set_bool("prompt_for_logout",
-				      global_config.prompt_for_logout);
 		gnome_config_set_bool("disable_animations",
 				      global_config.disable_animations);
 		gnome_config_set_int("applet_padding",
@@ -624,6 +623,9 @@ load_default_applets(void)
 	/*guess redhat menus*/
 	if(g_file_exists("/etc/X11/wmconfig"))
 		flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
+	/*guess KDE menus */
+	if(g_file_exists(KDE_MENUDIR))
+		flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
 	/*guess debian menus*/
 	if (g_file_exists("/etc/menu-methods/gnome"))
 		flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
@@ -737,6 +739,9 @@ init_user_applets(void)
 				/*guess redhat menus*/
 				if(g_file_exists("/etc/X11/wmconfig"))
 					flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
+				/*guess KDE menus */
+				if(g_file_exists(KDE_MENUDIR))
+					flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
 				/*guess debian menus*/
 				if (g_file_exists("/etc/menu-methods/gnome"))
 					flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
@@ -957,9 +962,6 @@ load_up_globals(void)
 	global_config.off_panel_popups =
 		gnome_config_get_bool("off_panel_popups=TRUE");
 		
-	global_config.prompt_for_logout =
-		gnome_config_get_bool("prompt_for_logout=TRUE");
-
 	global_config.disable_animations =
 		gnome_config_get_bool("disable_animations=FALSE");
 		
